package parser;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import model.Actor;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

import parser.sax.SAXSceneParserHandler;

/**
 * Prototypische Klasse, in der eine <Scene>-XML-Datei nach
 * der SAX-API geparst wird
 * 
 * @author Erik Wittkorn
 */
public class SAXSceneParser {
	
	/**
	 * Dieser Funktion wird der Name einer passenden XML-Datei uebergeben und
	 * diese wird nach der SAX-API geparst
	 * @param fileName Name der XML-Datei
	 * @return Liste mit Actors, die der XML entsprechen
	 */
	public static ArrayList<Actor> parse(String fileName) {
		ArrayList<Actor> results = new ArrayList<Actor>();
		try {
			// Die XML-Datei wird eingelesen
			FileReader reader = new FileReader(fileName);
			InputSource inputSource = new InputSource(reader);
			
			// Der XMLReader wird erstellt
			XMLReader xmlReader = XMLReaderFactory.createXMLReader();
			// Dem Handler-Objekt wird die Liste fuer die Ergebnisse ubergeben
			SAXSceneParserHandler actorHandler = new SAXSceneParserHandler(results);
			xmlReader.setContentHandler(actorHandler);
			
			// Die XML-Datei wird geparst
			xmlReader.parse(inputSource);
		
		} catch (SAXException | IOException e) {
			e.printStackTrace();
		}

		return results;
	}
}
