package components;

import java.util.HashMap;
import java.util.Set;

import logging.Logger;

import org.w3c.dom.Node;

import util.XMLUtil;
import actor.ActorComponent;

/**
 * Komponente, die eine beliebige Anzahl von Attributen haelt
 * 
 * @author Erik Wittkorn
 */
public class AttributeContainerComponent extends ActorComponent {
	
	/**
	 * Eindeutige ID der Komponente
	 */
	public static final int ID = 0xba0be6cb;

	/**
	 * HashMap fuer Attribute, in der fuer den Namen des Attributs ein Integerwert hinterlegt ist
	 */
	private HashMap<String,Integer> attributeValueMap = new HashMap<String,Integer>();

	@Override
	protected int getComponentId() {
		return ID;
	}

	@Override
	protected boolean init(Node data) {
		// Die Attribute werden von der ActorComponent-Node in die HashMap eingelesen
		while(data != null) {
            switch(data.getNodeName()) {
                case "Attribute":
                    String name = XMLUtil.getAttributeAsString(data, "name");
                    int value = XMLUtil.getAttributeAsInt(data, "value", 0);
                    attributeValueMap.put(name, value);
                    break;
                default:
                    break;
            }
            data = data.getNextSibling();
        }
		return true;
	}
	
	/**
	 * Gibt den aktuellen Stand eines Attributs zurueck
	 * @param name Name des Attributs
	 * @return Wert des Attributs
	 */
	public Integer getValueForAttribute(String name) {
		Integer result = attributeValueMap.get(name);
		if (result == null) {
			Logger.INSTANCE.warning("Achtung, Komponente " + name + " wurde beim Actor " + this.getOwner().getName() + " nicht gefunden");
		}
		return result;
	}
	
	/**
	 * Addiert einen bestimmten Wert zu einem Attribut
	 * @param name Name des Attributs
	 * @param value Wert, der hinzugefuegt werden soll
	 */
	public void addValueToAttribute(String name, int value) {
		int currValue = this.getValueForAttribute(name);
		currValue += value;
		attributeValueMap.put(name, currValue);
	}
	
	/**
	 * Gibt alle Namen der Attribute dieses Containers zurueck
	 * @return Ungeordnetes Set der Attributnamen
	 */
	public Set<String> getAttributeNames() {
		return this.attributeValueMap.keySet();
	}
	

}
