package processes;

import java.util.HashMap;

import main.GameApp;
import process.EngineProcess;
import actor.Actor;

import components.AttributeContainerComponent;

import events.LogicEvents;

/**
 * Dieser Prozess kann die Attributwerte eines Pickups zu denen eines Spielers hinzufuegen
 * 
 * @author Erik Wittkorn
 */
public class PickupProcess extends EngineProcess {

	/**
	 * Referenzen auf die AttributeContainerComponent-Komponente des Spielers
	 * und des Pickups
	 */
	private final AttributeContainerComponent playerCont;
	private final AttributeContainerComponent pickupCont;
	
	/**
	 * Hier wird hinterlegt, wie viel noch auf welches Attribut des Spielers addiert werden muss
	 */
	private final HashMap<String, Integer> attributeToGoalValueMap = new HashMap<String, Integer>();
	
	/**
	 * Zeit-Zaehler
	 */
	long currTime = 0;
	
	public PickupProcess(Actor player, Actor pickup) {
		pickupCont = (AttributeContainerComponent) pickup.getComponent(AttributeContainerComponent.ID);
		playerCont = (AttributeContainerComponent) player.getComponent(AttributeContainerComponent.ID);
		
		for (String attribute : pickupCont.getAttributeNames()) {
			Integer value = playerCont.getValueForAttribute(attribute);
			if (value != null) {
				// Die HashMap wird fuer alle Attribute, die beim Spieler und Pickup vorhanden sind
				// mit dem Wert des Pickups gefuellt
				this.attributeToGoalValueMap.put(attribute, pickupCont.getValueForAttribute(attribute));
			}
		}
		
	}
	
	
	@Override
	/**
	 * Die Methode wird in jedem Frame ausgefuehrt und erhaelt die vergangene Zeit als Parameter
	 */
	protected void onUpdate(long dMillis) {
		// Zeit wird auf Zaehlvariable addiert
		currTime += dMillis;
		// Wenn 1000 Millisekunden vergangen sind
		if (currTime > 50) {
			currTime -= 1000;
			// Werden die zu veraendernden Attribute durchlaufen
			for (String attribute : this.attributeToGoalValueMap.keySet()) {
				Integer value = this.attributeToGoalValueMap.get(attribute);
				// Wenn noch etwas zu addieren uebrig ist
				if (value > 0) {
					value--;
					// Wird dem Player-Attribut 1 hinzugefuegt
					playerCont.addValueToAttribute(attribute, 1);
					// Und der dekrementierte Wert in die HashMap geschrieben
					this.attributeToGoalValueMap.put(attribute, value);
				} else {
					// Wenn nichts mehr zu addieren ist, wird ein Event geworfen, das den Pickup-Actor loescht
					//TODO: Quest 4
					this.succeed();
				}
			}
		}
	}

}
