package util;

import java.io.IOException;
import java.io.OutputStream;
import java.util.Random;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import logging.Logger;
import logic.MageQuestLogic;
import model.Level;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Hilfsklasse, um Actors zu erstellen
 * 
 * @author Erik Wittkorn
 */
public class MageQuestUtil {

	/**
	 * Erstellt aufgrund der Level-Informationen die Actors in der GameLogic
	 */
	public static void createActorsFromLevel() {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		// Hilfsvariablen fuer Informationen des Levels
		int xOffset = MageQuestLogic.level.getxOffset();
		int yOffset = MageQuestLogic.level.getyOffset();
		int zOffset = MageQuestLogic.level.getzOffset();
		int xSize = MageQuestLogic.level.getxSize();
		int ySize = MageQuestLogic.level.getySize();
		int zSize = MageQuestLogic.level.getzSize();
		int tileSize = MageQuestLogic.level.getTileSize();
		int [][][] elements = MageQuestLogic.level.getElements();
		
		
		try {
			// Klassen zur Erstellung von XML-Nodes
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			Node sceneNode = document.createElement("Scene");
			document.appendChild(sceneNode);
			
			// Die Elemente des Arrays werden in allen Dimensionen durchlaufen
			for (int xPos = 0; xPos < xSize; xPos++) {
				for (int yPos = 0; yPos < ySize; yPos++) {
					for (int zPos = 0; zPos < zSize; zPos++) {
						// Der Typ des aktuellen Elements wird aus der HashMap gelesen
						String type = Level.idToBlocktype.get(elements[xPos][yPos][zPos]);
						if (type != null) {
							// Wenn der Typ mit "cube" startet
							if (type.startsWith("cube")) {
								// Wird eine Cube-Actor Node mit den benoetigten Komponenten generiert
								Element cubeActorNode = null;
								cubeActorNode = document.createElement("Actor");
								cubeActorNode.setAttribute("resource", "actors/CubeActor.xml");
								cubeActorNode.setAttribute("name", "Cube_" + xPos + "_" + yPos + "_" + zPos);
								
								Element transCompNode = document.createElement("TransformComponent");
								Element positionNode = document.createElement("Position");
								float x = xOffset + tileSize * xPos;
								float y = yOffset + tileSize * yPos;
								float z = zOffset + tileSize * zPos;
								//Der TransformKomponente wird die aktuelle Position ueberliefert
								positionNode.setAttribute("x", String.valueOf(x));
								positionNode.setAttribute("y", String.valueOf(y));
								positionNode.setAttribute("z", String.valueOf(z));
								transCompNode.appendChild(positionNode);
								cubeActorNode.appendChild(transCompNode);
								
								Element renderCompNode = document.createElement("RenderComponent");
								Element objMeshNodeNode = document.createElement("ObjMesh");
								objMeshNodeNode.setAttribute("resource", "meshes/cube.obj");
								
								// In der Renderkomponente wird das Material angegeben, das dem aktuellen Typ entspricht
								objMeshNodeNode.setAttribute("customMaterialLibrary", "meshes/" + type + ".mtl");
								renderCompNode.appendChild(objMeshNodeNode);
								cubeActorNode.appendChild(renderCompNode);
								
								// Die SzeneNode ist nur fuer Debugging Ausgaben da
								sceneNode.appendChild(cubeActorNode);
								
								// Der Actor wird mit der Node erstellt (nicht vergessen!)
								int actorID = MageQuestLogic.createActor(cubeActorNode).getId();
								// Und die ActorID wird im Level hinterlegt (nicht vergessen!)
								MageQuestLogic.level.setActorID(xPos, yPos, zPos, actorID);
							}
							// Wenn der Typ mit "Leaves" startet, wird eine Baumkrone generiert
							if (type.startsWith("leaves")) {
								// TODO: Quest 2
							}
							// Wenn der Typ mit "pickup" startet, wird die Hilfsfunktion zur Generierung eines neuen Pickups genutzt
							if (type.startsWith("pickup")) {
								createPickupActor(xPos, yPos, zPos);
							}
						}
					}
				}
			}
			
//			Debug Ausgabe der gesamten generierten XML, Achtung, viel
//			try {
//				XMLUtil.printDocument(document, (OutputStream)System.out);
//			} catch (IOException | TransformerException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
			
		} catch (ParserConfigurationException parserException) {
			parserException.printStackTrace();
		}
	}

	/**
	 * Hilsmethode, erstellt einen zufaellig bestimmten Pickup-Actor an gewuenschter Stelle (in Levelkoordinaten)
	 * @param xPos x-Position
	 * @param yPos y-Positin
	 * @param zPos z-Position
	 */
	public static void createPickupActor(int xPos, int yPos, int zPos) {
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();
			Document document = builder.newDocument();
			Element pickupActorNode = null;
			
			Random r = new Random();
			// Eine zufaellige Zahl zwischen 1 und 0, um zufaellig Mana- oder Health-Pickups zu erstellen
			int pickupType = r.nextInt(2);
			
			// Transformkomponente bekommt die richtige Position

			// Renderkomponente bekommt je nach Zufallswert ein rotes, oder blaues Material
			
			// PickupActorNode bekommt eine AttributeContainerKomponente, die je nach Zufallswert passend zur Farbe etwas Mana, oder Health hat
			
			// PickupActorNode bekommt ein PickupComponent
			
			// Der Actor fuer dieses Pickup wird erstellt
			// Die ActorID wird im passenden Array hinterlegt
			// Das elementsArray wird an der richtigen Stelle aktualisiert und wieder hinterlegt,
			// dass sich dort ein Pickup befindet (id 6)
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
}
