package parser;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import model.RectangleShape;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * Prototypische Klasse, deren Funktion eine XML-Datei mit
 * einem RectangleShape nach dem DOM-Modell parst
 */
public class DOMRectangleShapeParser {

	/**
	 * Parsing-Funktion, die die XML nach dem DOM-Modell parst
	 * @param filename Dateiname der XML-Datei
	 * @return RectangleShape-Objekt mit den in der XML definierten Eigenschaften
	 */
	public static RectangleShape parse(String filename) {
		RectangleShape rectangleShape = null;
		
		try {
			// Die Datei wird refrenziert
			File xmlFile = new File(filename);
			
			// Die Objekte fuer das DOM-Parsing werden initialisiert
			DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
			
			// Aus dem Parsing entsteht ein Document-Object, dessen erste Kind-Node als CubeShape-Node eingelesen wird
			Document xmlDoc = dBuilder.parse(xmlFile);
			Node rectangleShapeNode = xmlDoc.getFirstChild();
			
			while (rectangleShapeNode != null) {
				// Ein neues CubeShape-Objekt wird erstellt und die erste Kind-Node ausgelesen
				rectangleShape = new RectangleShape();
				Node rectangleShapeChildNode = rectangleShapeNode.getFirstChild();
				
				while (rectangleShapeChildNode != null) {
					// Je nach Tag-Name werden die Nodes anders verarbeitet
					switch (rectangleShapeChildNode.getNodeName()) {
						case "Position":
							float x = Float.parseFloat(rectangleShapeChildNode.getAttributes().getNamedItem("x").getNodeValue());
							float y = Float.parseFloat(rectangleShapeChildNode.getAttributes().getNamedItem("y").getNodeValue());
							rectangleShape.setPosition(x, y);
							break;
							
						case "Size":
							float width = Float.parseFloat(rectangleShapeChildNode.getAttributes().getNamedItem("width").getNodeValue());
							float height = Float.parseFloat(rectangleShapeChildNode.getAttributes().getNamedItem("height").getNodeValue());
							rectangleShape.setSize(width, height);
							break;
						
						case "Color":
							String color = rectangleShapeChildNode.getTextContent();
							rectangleShape.setColor(color);
							break;
							
						default:
							//Nothing to do here
							break;
					}
					// Die "Geschwister"-Node wird eingelesen
					rectangleShapeChildNode = rectangleShapeChildNode.getNextSibling();
				}
				// Die "Geschwister"-Node wird eingelesen
				rectangleShapeNode = rectangleShapeNode.getNextSibling();
			}
			
		} catch (ParserConfigurationException | SAXException | IOException e) {
			e.printStackTrace();
		}
		
		return rectangleShape;
	}
}
