package parser.sax;

import model.RectangleShape;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Handler-Klasse, die die Events verarbeitet, die beim SAX-Parsing
 * einer XML-Datei mit <RectangleShape>-Elementen auftreten
 */
public class SAXRectangleShapeParserHandler implements ContentHandler {
	
	/**
	 * Platzhalter-Referenzen fuer die Objekte, die durch die
	 * XML erstellt werden
	 */
	private final RectangleShape rectangleShape;
	
	private String currentString = null;

	
	public SAXRectangleShapeParserHandler(RectangleShape cubeShape) {
		this.rectangleShape = cubeShape;
	}
	
	@Override
	public void startDocument() throws SAXException {
		
	}

	/**
	 * Wird aufgerufen, wenn ein Start-Tag beim Parsing vorkommt
	 */
	@Override
	public void startElement(String uri, String localName, String qName,
			Attributes atts) throws SAXException {
		switch (localName) {
		
				// Die Position der Form wird aus den Attributen des <Position>-Tags ausgelesen
			case "Position":
				float x = Float.parseFloat(atts.getValue("x"));
				float y = Float.parseFloat(atts.getValue("y"));
				rectangleShape.setPosition(x, y);
				break;
				
				// Die Groesse der Form wird aus den Attributen des <Size>-Tags ausgelesen
			case "Size":
				float width = Float.parseFloat(atts.getValue("width"));
				float height = Float.parseFloat(atts.getValue("height"));
				rectangleShape.setSize(width, height);
				break;
			
				// Damit auf jeden Fall eine Farbe gesetzt ist, wird beim
				// Start-Tag <Color> eine default-Farbe gesetzt
			case "Color":
				currentString = "defaultColor";
				break;
				
			default:
				// Nothing to do here...
				break;
		}
	}

	/**
	 * Diese Methode wird aufgerufen, wenn ein End-Tag beim XML-Parsing
	 * vorgekommen ist
	 */
	@Override
	public void endElement(String uri, String localName, String qName)
			throws SAXException {
		switch (localName) {
			
			// Der Name der Farbe muesste sich in der aktuellen String-Variable befinden
			// und wird gesetzt
			case "Color":
				if (rectangleShape != null) {
					rectangleShape.setColor(currentString);
				} else {
					System.out.println("Achtung, RectangleShape ist null");
				}
				break;
				
			default:
				//Nothing to do here...
				break;
		}
	}

	/**
	 * Der Content eines Elements wird eingelesen und in einer
	 * String-Referenz abgelegt
	 */
	@Override
	public void characters(char[] ch, int start, int length) throws SAXException {
		currentString = new String(ch, start, length);
	}

	@Override
	public void endDocument() throws SAXException {
		
	}
	
	
	
	
	

	@Override
	public void endPrefixMapping(String arg0) throws SAXException {
	}

	@Override
	public void ignorableWhitespace(char[] arg0, int arg1, int arg2)
			throws SAXException {
	}

	@Override
	public void processingInstruction(String arg0, String arg1)
			throws SAXException {
	}

	@Override
	public void setDocumentLocator(Locator arg0) {
	}

	@Override
	public void skippedEntity(String arg0) throws SAXException {
	}

	@Override
	public void startPrefixMapping(String arg0, String arg1)
			throws SAXException {
	}
}
