package logic;

import input.InputListener;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import logging.Logger;
import main.BaseGameLogic;
import main.GameApp;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import util.Pose;
import util.XMLUtil;
import actor.Actor;
import actor.components.TransformComponent;
import events.ControlEvents;

public class TestBedLogic extends BaseGameLogic {

	public static final int DIR_NO_DIRECTION = 0;
    public static final int DIR_POS_X = 1;
    public static final int DIR_NEG_X = 2;
    public static final int DIR_POS_Z = 3;
    public static final int DIR_NEG_Z = 4;
    
    public static Actor player = null;
    public static Pose playerPose = null;
    
    public static InputListener input = new InputListener();
	
	@Override
	public boolean init() {
		// TODO Auto-generated method stub
		boolean success = super.init();
		GameApp.getEventManager().register(input,
											ControlEvents.ForwardEvent.ID,
											ControlEvents.BackwardEvent.ID,
											ControlEvents.LeftEvent.ID,
											ControlEvents.RightEvent.ID,
											ControlEvents.UpEvent.ID,
											ControlEvents.DownEvent.ID,
											ControlEvents.LeftwardEvent.ID,
											ControlEvents.RightwardEvent.ID,
											ControlEvents.EnterEvent.ID);
		return success;
	}
    
	@Override
	public void loadLevel(String resource) {
        Document doc = GameApp.loadXML(resource);
        Node node = doc.getFirstChild();
        while(node != null && !node.getNodeName().equals("Actors")) {
            node = node.getNextSibling();
        }
        if(node != null) {
            Node actorNode = node.getFirstChild();
            while(actorNode != null) {
                if(actorNode.getNodeName().equals("Actor")) {
                    String name = XMLUtil.getAttributeAsString(actorNode, "name");

                    Actor currentActor = this.createActor(XMLUtil.getAttributeAsString(actorNode, "resource"), actorNode);
                    
                    if (name.equals("player")) {
                    	this.player = currentActor;
                    	TransformComponent transComp = (TransformComponent)this.player.getComponent(TransformComponent.ID);
                    	this.playerPose = transComp.getPose();
                    }
                }
                actorNode = actorNode.getNextSibling();
            }
        } else {
            Logger.INSTANCE.warning("Level file does not contain any actors.");
        }
        
        TestBedLogic.createMeadow(4, 4);
	}
	
	public static void createMeadow(int width, int length) {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
	    int tileSize = 1;
	    try {
	      DocumentBuilder builder = factory.newDocumentBuilder();
	      Document document = builder.newDocument();

	      for (int xPos = 0; xPos < width; xPos++) {
	    	  for (int zPos = 0; zPos < length; zPos++) {
	    		  Element grassCubeActorNode = null;
	    		  grassCubeActorNode = document.createElement("Actor");
	    		  grassCubeActorNode.setAttribute("resource", "actors/CubeActor.xml");
	    		  grassCubeActorNode.setAttribute("name", "GrassCube_" + xPos + "_" + zPos);
	    		  
	    		  Element transCompNode = document.createElement("TransformComponent");
	    		  Element positionNode = document.createElement("Position");
	    		  float x = tileSize * xPos;
	    		  float z = tileSize * zPos;
	    		  positionNode.setAttribute("x", String.valueOf(x));
	    		  positionNode.setAttribute("z", String.valueOf(z));
	    		  positionNode.setAttribute("y", "0.0");
	    		  transCompNode.appendChild(positionNode);
	    		  grassCubeActorNode.appendChild(transCompNode);
	    		  
	    		  Element renderCompNode = document.createElement("RenderComponent");
	    		  Element objMeshNodeNode = document.createElement("ObjMesh");
	    		  objMeshNodeNode.setAttribute("resource", "meshes/cube.obj");
	    		  objMeshNodeNode.setAttribute("customMaterialLibrary", "meshes/cube_grass.mtl");
	    		  renderCompNode.appendChild(objMeshNodeNode);
	    		  grassCubeActorNode.appendChild(renderCompNode);
	    		  
	    		  TestBedLogic.createActor(grassCubeActorNode);
	    	  }
	      }

	    } catch (ParserConfigurationException parserException) {
	      parserException.printStackTrace();
	    }
	}
	
    /**
     * Erzeugt einen Actor aus einer Node einer xml-Leveldatei.
     * @param node Die Node des Actors
     */
    public static Actor createActor(Node node) {
        String resource = XMLUtil.getAttributeAsString(node, "resource");
        Actor actor = null;
        if(resource != null) {
            actor = GameApp.getGameLogic().createActor(resource, node);
        }
        return actor;
    }
    
    public static void movePlayer(int direction) {
    	int xPos = (int) TestBedLogic.playerPose.getPosition().x;
    	int yPos = (int) TestBedLogic.playerPose.getPosition().y;
    	int zPos = (int) TestBedLogic.playerPose.getPosition().z;
    	switch (direction) {
			case TestBedLogic.DIR_NEG_Z:
				zPos --;
				break;
			case TestBedLogic.DIR_NEG_X:
				xPos --;
				break;
			case TestBedLogic.DIR_POS_X:
				xPos ++;
				break;
			case TestBedLogic.DIR_POS_Z:
				zPos ++;
			default:
				break;
		}
    	playerPose.setPosition(xPos, yPos, zPos);
    }

}
