package model;

import java.util.HashMap;
import java.util.Map;

import model.components.ActorComponent;

/**
 * Prototypische Klasse fuer einen Actor
 * mit Komponenten und einem Namen
 * 
 * @author Erik Wittkorn
 */
public class Actor {
	
	/**
	 * Name des Actors
	 */
	private final String name;

	/**
	 * HashMap mit den Komponenten als Werte und
	 * derern Name als Schluessel
	 */
	private final Map<String, ActorComponent> components = new HashMap<String, ActorComponent>();
	
	public Actor(String name) {
		this.name = name;
	}
	
	public String getName()	{
		return this.name;
	}
	
	/**
	 * Beim Hinzufuegen eines Komponenten wird automatisch
	 * der Name als Schluessel fuer die HashMap verwendet und
	 * der Besitzer der Komponente gesetzt
	 * @param comp Komponente, die dem Actor hinzugefuegt werden soll
	 */
	public void addComponent(ActorComponent comp) {
		this.components.put(comp.getName(), comp);
		comp.setOwner(this);
	}
	
	/**
	 * Man kann eine Komponente mit ihrem Namen wieder
	 * erhalten
	 * @param name Name der Komponente
	 * @return Komponenten-Instanz des Actors
	 */
	public ActorComponent getComponent(String name) {
		return components.get(name);
	}
	
	/**
	 * Gibt alle verfuegbaren Informationen ueber den Actor als String
	 * zurueck, wie auch eine Liste seiner Komponenten
	 */
	public String toString() {
		String description = "Actor with Name: " + this.name + " and ComponentCount: " + components.size();
		if (this.components.size() > 0) {
			description += "\n----- ActorComponents start -----\n";
			for (ActorComponent actorComp : components.values()) {
				description += "\n" + actorComp;
			}
			description += "\n----- ActorComponents end -----\n";
		}
		return description;
	}
}
