package model.components;

import java.util.ArrayList;

import model.BoundingSphere;

/**
 * Prototypische Komponente um die visuellen Eigenschaften eines
 * Actors zu beschreiben
 * 
 * Ist von der allgemeinen Actor-Komponente abgeleitet
 * 
 * @author Erik Wittkorn
 */
public class RenderComponent extends ActorComponent {
	
	/**
	 * Beinhaltet den Namen der Ressource, in der das Mesh
	 * fuer den sichtbaren Meshs fuer den Actor beschrieben ist
	 */
	private String objMesh = null;
	
	/**
	 * In dieser Liste ist durch BoundingSpheres eine umgebene Huelle
	 * des Actors definiert, die fuer Sichtbarkeitstests genutzt wird
	 */
	ArrayList<BoundingSphere> spheres = new ArrayList<BoundingSphere>();
	
	/**
	 * Im Konstruktor wird der eindeutige Name dieser Komponente eingetragen
	 * und die Owner-Referenz auf null gesetzt
	 */
	public RenderComponent() {
		this.setName("RenderComponent");
		this.setOwner(null);
		
		this.objMesh = "mesh/default.obj";
	}
	
	public void setObjMesh(String objMesh) {
		this.objMesh = objMesh;
	}
	
	public String getObjMesh() {
		return this.objMesh;
	}
	
	public void addBoundingSphere(BoundingSphere bSphere) {
		this.spheres.add(bSphere);
	}
	
	public ArrayList<BoundingSphere> getBoundingSpheres() {
		return this.spheres;
	}

	/**
	 * Siehe Oberklasse
	 */
	@Override
	public String toString() {
		String description = "RenderComponent with objMesh: " + this.objMesh + " and SphereCount: " + this.spheres.size();
		if (this.spheres.size() > 0) {
			description += "\n----- BoundingSpheres start -----";
			for (BoundingSphere bSphere : spheres) {
				description += "\n" + bSphere;
			}
			description += "\n----- BoundingSpheres end -----\n";
		}
		return description;
	}

}
