package model.components;

import org.lwjgl.util.vector.Vector3f;

/**
 * Prototypische Komponente um die Eigenschaften der Transformation
 * der Position eines Actors zu beschreiben
 * 
 * Ist von der allgemeinen Actor-Komponente abgeleitet
 * 
 * @author Erik Wittkorn
 */
public class TransformComponent extends ActorComponent {
	
	/**
	 * In diesem Vektor ist die Position des Actors
	 * im dreidimensionalen Raum als Koordinaten
	 * angegeben
	 */
	Vector3f position = null;
	
	/**
	 * In diesem Vektor ist die Rotation des Actors
	 * im dreidimensionalen angegeben, wobei die
	 * x-Koordinate dem Yaw-, die y-Koordinate dem
	 * Pitch- und die z-Koordinate dem Roll-Winkel entspricht
	 */
	Vector3f rotation = null;

	/**
	 * Im Konstruktor wird der eindeutige Name dieser Komponente eingetragen
	 * und die Owner-Referenz auf null gesetzt
	 * 
	 * Ausserdem wird die Position und die Rotation initial auf (0, 0, 0) gesetzt
	 */
	public TransformComponent() {
		this.setName("TransformComponent");
		this.setOwner(null);
		
		this.position = new Vector3f();
		this.rotation = new Vector3f();
	}
	
	public void setPosition(float x, float y, float z) {
		this.position.setX(x);
		this.position.setY(y);
		this.position.setZ(z);
	}
	
	public void setRotation(float yaw, float pitch, float roll) {
		this.rotation.setX(yaw);
		this.rotation.setY(pitch);
		this.rotation.setZ(roll);
	}
	
	public Vector3f getPosition() {
		return this.position;
	}
	
	public Vector3f getRotation() {
		return this.rotation;
	}

	/**
	 * Siehe Oberklasse
	 */
	@Override
	public String toString() {
		String description = "TransformComponent with Position: (" + this.position.x + ", " + this.position.y + ", " + this.position.z + ") and Rotation: (" + this.rotation.x + ", " + this.rotation.y + ", " + this.rotation.z + ")";
		description += '\n';
		return description;
	}
}
